@@ -36,7 +36,7 @@ from airflow.exceptions import (
 )
 from airflow.models.taskmixin import DAGNode
 from airflow.serialization.enums import DagAttributeTypes
-from airflow.utils.helpers import validate_group_key
+from airflow.utils.helpers import validate_group_key, validate_instance_args
 
 if TYPE_CHECKING:
     from sqlalchemy.orm import Session
@@ -49,6 +49,21 @@ if TYPE_CHECKING:
     from airflow.models.taskmixin import DependencyMixin
     from airflow.utils.edgemodifier import EdgeModifier
 
+# TODO: The following mapping is used to validate that the arguments passed to the TaskGroup are of the
+#  correct type. This is a temporary solution until we find a more sophisticated method for argument
+#  validation. One potential method is to use get_type_hints from the typing module. However, this is not
+#  fully compatible with future annotations for Python versions below 3.10. Once we require a minimum Python
+#  version that supports `get_type_hints` effectively or find a better approach, we can replace this
+#  manual type-checking method.
+TASKGROUP_ARGS_EXPECTED_TYPES = {
+    "group_id": str,
+    "prefix_group_id": bool,
+    "tooltip": str,
+    "ui_color": str,
+    "ui_fgcolor": str,
+    "add_suffix_on_collision": bool,
+}
+
 
 class TaskGroup(DAGNode):
     """
@@ -160,6 +175,8 @@ class TaskGroup(DAGNode):
         self.upstream_task_ids = set()
         self.downstream_task_ids = set()
 
+        validate_instance_args(self, TASKGROUP_ARGS_EXPECTED_TYPES)
+
     def _check_for_group_id_collisions(self, add_suffix_on_collision: bool):
         if self._group_id is None:
             return
@@ -209,7 +226,8 @@ class TaskGroup(DAGNode):
                 yield child
 
     def add(self, task: DAGNode) -> DAGNode:
-        """Add a task to this TaskGroup.
+        """
+        Add a task to this TaskGroup.
 
         :meta private:
         """
@@ -532,7 +550,8 @@ class TaskGroup(DAGNode):
         return graph_sorted
 
     def iter_mapped_task_groups(self) -> Iterator[MappedTaskGroup]:
-        """Return mapped task groups in the hierarchy.
+        """
+        Return mapped task groups in the hierarchy.
 
         Groups are returned from the closest to the outmost. If *self* is a
         mapped task group, it is returned first.
@@ -566,7 +585,8 @@ class TaskGroup(DAGNode):
 
 
 class MappedTaskGroup(TaskGroup):
-    """A mapped task group.
+    """
+    A mapped task group.
 
     This doesn't really do anything special, just holds some additional metadata
     for expansion later.
